<?php

namespace Imunify360\panels;


const JSON_CONTENT_TYPE = 'Content-type: application/json; charset=utf-8';

abstract class AbstractPanel {
    const MODE = 'prod';
    const FILE_MAX_SIZE = 2097152;
    const STATIC_PATH = 'assets/static/';

    const MANIFEST_PATH = self::MANIFEST_PATH; // Child class must override this

    /**
     * @var \stdClass
     */
    public $request;
    /**
     * @var bool
     */
    public $isAdmin = true;
    public $manifest = null;

    /**
     * @return void
     */
    public abstract function adminAction();

    /**
     * @return void
     */
    public abstract function userAction();

    /**
     * @return string
     */
    public abstract function getJson();

    /**
     * @param mixed $renderer
     * @return void
     */
    public abstract function renderAssets($renderer = null);

    /**
     * @param string $data
     * @param string $action
     * @return string
     * @throws \ErrorException
     */
    public abstract function execute($data, $action = 'execute');

    /**
     * AbstractPanel constructor.
     * @param bool $isAdmin
     */
    public function __construct($isAdmin = true)
    {
        $this->isAdmin = $isAdmin;
    }

    /**
     * @return array
     */
    public function getPost() {
        return $_POST;
    }

    /**
     *
     */
    public function defaultAction()
    {
        if ($this->isAdmin) {
            $this->adminAction();
        } else {
            $this->userAction();
        }
    }

    /**
     *
     */
    public function uploadFile()
    {
        $data = array(
            'files' => array(),
        );

        foreach ($_FILES['files']['name'] as $k => $v) {
            $data['files'][$_FILES['files']['tmp_name'][$k]] = $_FILES['files']['name'][$k];
        }

        try {
            $response = $this->execute(json_encode($data), 'uploadFile');
            $this->renderSuccess($response);
        } catch (\ErrorException $e) {
            $this->renderError($response);
        }
    }

    /**
     * @return string
     */
    public function prepareRequest() {
        $args = new \stdClass();
        $args->command = $this->request->method;

        $args->params = new \stdClass();
        if (isset($this->request->params)) {
            $args->params = $this->request->params;
        }
        $args->params->remote_addr = $_SERVER['REMOTE_ADDR'];

        return json_encode($args);
    }

    /**
     * @param string $data
     * @param bool $error
     */
    public function render($data, $error = false)
    {
        if ($error) {
            $this->renderError($data);
        } else {
            $this->renderSuccess($data);
        }
    }

    /**
     * @param string $data
     */
    public function renderSuccess($data)
    {
        header(JSON_CONTENT_TYPE);
        header('HTTP/1.1 200 OK');
        echo $data;
        exit(0);
    }

    /**
     * @param string $data
     */
    public function renderError($data)
    {
        $json = json_decode($data);
        if ($json && !isset($json->error)) {
            $this->renderSuccess($data);
        }

        header(JSON_CONTENT_TYPE);
        header('HTTP/1.1 502 Bad Gateway');
        echo $data;
        exit(1);
    }

    /**
     * @param $data
     */
    public function dump($data)
    {
        header(JSON_CONTENT_TYPE);
        header('HTTP/1.1 200 OK');
        var_dump($data);
        exit(0);
    }

    /**
     * @param string $command
     */
    public function commandHeader($command, $user = null)
    {
        if (self::MODE !== 'prod') {
            header('X-I360-COMMAND: ' . $this->escapeCommand($command));
            if ($user) {
                header('X-I360-USER: ' . $this->escapeCommand($user));
            }
        }
    }

    /**
     * @return array
     */
    public function getAssets() {
        $assets = array();
        $this->addAsset($assets, 'main.css');
        $assets[] = 'assets/js/config.js'; // if you changed config.js, add `?v=1`
        $this->addAsset($assets, 'polyfills.js');
        $this->addAsset($assets, 'vendor.js');
        $this->addAsset($assets, 'main.js');

        return $assets;
    }

    /**
     * @return StdClass
     */
    public function getManifest() {
        if (!$this->manifest) {
            $this->manifest = json_decode($this->getManifestContent());
        }

        return $this->manifest;
    }

    /**
     * @return string|boolean
     */
    public function getManifestContent() {
        return file_get_contents(static::MANIFEST_PATH);
    }

    /**
     * @param array $assets
     * @param StdClass $manifest
     * @param string $asset
     * @return array
     */
    protected function addAsset(&$assets, $asset) {
        $manifest = $this->getManifest();
        if (property_exists($manifest, $asset)) {
            $assets[] = self::STATIC_PATH . $manifest->{$asset};
        }

        return $assets;
    }

    /**
     * @param string $command
     * @return string
     */
    protected function escapeCommand($command)
    {
        return preg_replace('/[^[:alnum:][:punct:] ]+?/m', ' ', $command);
    }

    /**
     * @param string $value
     * @return string
     */
    protected function escapeParams($value)
    {
        return "'" . str_replace("'", "'\''", $value) . "'";
    }
}

